# coding=utf8
from pyramid_clamav import Tween
from io import BytesIO
import clamd
import mock
import os
import tempfile
import unittest


class FakeFileUpload(object):

    def __init__(self, content):
        _, tmp = tempfile.mkstemp()
        os.write(_, content)
        self.file = open(tmp, 'rb')


class StringIOFileUpload(object):

    def __init__(self, content):
        self.file = BytesIO(content)


class FakeRequest(object):

    def __init__(self, ctype, post):
        self.headers = {'Content-Type': ctype}
        self.POST = post


def fake_handler(arg):
    pass


clean_post = dict(uploadedfile=FakeFileUpload(b'clean file'))
clean_request = FakeRequest('multipart/form-data', clean_post)


class TestClam(unittest.TestCase):

    def test_request_with_no_vir_goes_through(self):
        with mock.patch('pyramid_clamav.tests.fake_handler') as fake_handler,\
                mock.patch('pyramid_clamav.handle_virus') as virus:
            tween = Tween(fake_handler, {})
            tween(clean_request)
            virus.assert_not_called()
            fake_handler.assert_called_with(clean_request)

    def test_vir_is_found_in_req(self):
        self.assert_virus_in_file_like_upload(FakeFileUpload(clamd.EICAR))

    def test_vir_is_found_in_req_with_stringio(self):
        self.assert_virus_in_file_like_upload(StringIOFileUpload(clamd.EICAR))

    def assert_virus_in_file_like_upload(self, file_like):
        post = dict(uploadedfile=file_like)
        request = FakeRequest('multipart/form-data', post)

        with mock.patch('pyramid_clamav.tests.fake_handler'), \
                mock.patch('pyramid_clamav.handle_virus') as virus:
            tween = Tween(fake_handler, {})
            tween(request)
            virus.assert_called_with(request,
                                     'Eicar-Test-Signature',
                                     'uploadedfile')
            fake_handler.assert_called_with(request)
            assert request.POST['uploadedfile'].file.read() == b''

    def test_no_failure_if_clamav_is_busy(self):
        with mock.patch('clamd.ClamdUnixSocket.instream') as instream:
            instream.side_effect = OSError('I am busy')
            tween = Tween(fake_handler, {})
            tween(clean_request)



from __future__ import unicode_literals
import clamd
import logging
import sys
import time
from pyramid.i18n import TranslationStringFactory, get_localizer
_ = TranslationStringFactory('pyramid_clamav')


clamlog = logging.getLogger(__name__)


def handle_virus(request, sig, key):
    localizer = get_localizer(request)
    log_message = 'Virus found: "{}"; Request key "{}"'.format(sig, key)
    clamlog.error(log_message)
    request.session.flash(
        localizer.translate(
            _('found-virus-user-message',
              default=('Virus found in file upload: ${sig}. '
                       'The file has been removed from the request.'),
              mapping={'sig': sig})),
        'error')
    del request.POST[key]


class Tween(object):

    scanning = True

    def __init__(self, handler, config):
        self.handler = handler
        self.config = config
        clamd_debug = config.get('pyramid_clamav.debug', 'False')
        if clamd_debug.lower().strip() in ('false', '0', 'no'):
            clamd_debug = False
        if sys.platform == 'darwin':
            self.clamd = clamd.ClamdUnixSocket('/tmp/clamd.socket')
        else:
            self.clamd = clamd.ClamdUnixSocket()
        if clamd_debug:
            try:
                self.clamd.ping()
            except clamd.ConnectionError:
                self.scanning = False
        if not self.scanning:
            clamlog.warn('Virus scanning deactivated. (pyramid_clamav.debug '
                         'is true and clamav not properly configured!')
        else:
            clamlog.info('Found clamd and its responding. Virus scanning '
                         'activated.')

    def _check_file(self, request, key, checks=0):
        try:
            return self.clamd.instream(request.POST[key].file)
        except (OSError, BrokenPipeError, clamd.ConnectionError):
            checks += 1
            if checks == 3:
                raise
            time.sleep(0.5)
            request.POST[key].file.seek(0)
            return self._check_file(request, key, checks)

    def __call__(self, request):
        if request.headers.get('Content-Type', '').startswith(
                'multipart/form-data'):
            for key in request.POST.keys():
                if hasattr(request.POST[key], 'file') and \
                       self.is_file_like(request.POST[key].file):
                    if not self.scanning:
                        localizer = get_localizer(request)
                        request.session.flash(
                            localizer.translate(
                                _('clamav-not-configured-message',
                                  default=(
                                      'File upload found but clamav is not '
                                      'configured.'
                                    )
                                  )
                                ), 'error')
                        clamlog.error('File upload found but '
                                      'clamav is not configured.')
                        continue
                    try:
                        result = self._check_file(request, key)
                    except (
                        OSError, BrokenPipeError, clamd.ConnectionError
                    ) as e:
                        clamlog.error(
                            'Connection to ClamD was lost: {}'.format(str(e))
                        )
                        request.POST[key].file.seek(0)
                    else:
                        if result and result.get('stream')[0] == 'FOUND':
                            sig = result.get('stream')[1]
                            handle_virus(request, sig, key)
                        else:
                            request.POST[key].file.seek(0)
        return self.handler(request)

    def is_file_like(self, obj):
        return hasattr(obj, 'read') and hasattr(obj, 'seek')


def factory(handler, registry):
    return Tween(handler, registry.settings.copy())


def includeme(config):
    config.add_tween('pyramid_clamav.factory')



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
try:
    README = open(os.path.join(here, 'README.rst')).read()
    CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()
except IOError:
    README = CHANGES = ''

install_requires = [
    'pyramid>=1.3',
    'mock',
    'clamd'
    ]

setup(name='pyramid_clamav',
      version='0.7',
      description=('A tween checking file uploads for viruses using clamav.'),
      long_description=README + '\n\n' + CHANGES,
      classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Framework :: Pyramid",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "License :: Repoze Public License",
        ],
      keywords='wsgi pylons pyramid clamav virus',
      author=("Daniel Havlik",),
      author_email="dh@gocept.com",
      url="http://www.gocept.com",
      license="BSD",
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=install_requires,
      entry_points='',
      )
